@@ -7,20 +7,29 @@ import time
 from enum import IntEnum, auto, unique
 from typing import List, Tuple
 
-if platform.system() == "Windows":
-    import ctypes
-
 import click
 import cv2
 import numpy as np
 from pydantic import ValidationError
+from tqdm import tqdm
 
 from .commons import config_path_option
 from .config import Config, PresentationConfig, SlideConfig, SlideType
 from .defaults import CONFIG_PATH, FOLDER_PATH, FONT_ARGS
 
+INTERPOLATION_FLAGS = {
+    "nearest": cv2.INTER_NEAREST,
+    "linear": cv2.INTER_LINEAR,
+    "cubic": cv2.INTER_CUBIC,
+    "area": cv2.INTER_AREA,
+    "lanczos4": cv2.INTER_LANCZOS4,
+    "linear-exact": cv2.INTER_LINEAR_EXACT,
+    "nearest-exact": cv2.INTER_NEAREST_EXACT,
+}
+
 WINDOW_NAME = "Manim Slides"
 WINDOW_INFO_NAME = f"{WINDOW_NAME}: Info"
+WINDOWS = platform.system() == "Windows"
 
 
 @unique
@@ -37,7 +46,7 @@ class State(IntEnum):
 
 
 def now() -> int:
-    """Returns time.time() in seconds."""
+    """Returns time.time() in milliseconds."""
     return round(time.time() * 1000)
 
 
@@ -55,6 +64,7 @@ class Presentation:
 
         self.current_slide_index = 0
         self.current_animation = self.current_slide.start_animation
+        self.current_file = None
 
         self.loaded_animation_cap = -1
         self.cap = None  # cap = cv2.VideoCapture
@@ -104,6 +114,8 @@ class Presentation:
                 file = "{}_reversed{}".format(*os.path.splitext(file))
                 self.reversed_animation = animation
 
+            self.current_file = file
+
             self.cap = cv2.VideoCapture(file)
             self.loaded_animation_cap = animation
 
@@ -196,6 +208,11 @@ class Presentation:
         else:
             return self.next_animation == self.current_slide.end_animation
 
+    @property
+    def current_frame_number(self) -> int:
+        """Returns current frame number."""
+        return int(self.current_cap.get(cv2.CAP_PROP_POS_FRAMES))
+
     def update_state(self, state) -> Tuple[np.ndarray, State]:
         """
         Updates the current state given the previous one.
@@ -252,14 +269,22 @@ class Display:
         start_paused=False,
         fullscreen=False,
         skip_all=False,
-        resolution=(1280, 720),
+        resolution=(1980, 1080),
+        interpolation_flag=cv2.INTER_LINEAR,
+        record_to=None,
     ):
         self.presentations = presentations
         self.start_paused = start_paused
         self.config = config
         self.skip_all = skip_all
         self.fullscreen = fullscreen
-        self.is_windows = platform.system() == "Windows"
+        self.resolution = resolution
+        self.interpolation_flag = interpolation_flag
+        self.record_to = record_to
+        self.recordings = []
+        self.window_flags = (
+            cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_FREERATIO | cv2.WINDOW_NORMAL
+        )
 
         self.state = State.PLAYING
         self.lastframe = None
@@ -274,43 +299,20 @@ class Display:
             cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_FREERATIO | cv2.WINDOW_AUTOSIZE,
         )
 
-        if self.is_windows:
-            user32 = ctypes.windll.user32
-            self.screen_width, self.screen_height = user32.GetSystemMetrics(
-                0
-            ), user32.GetSystemMetrics(1)
-
         if self.fullscreen:
-            cv2.namedWindow(WINDOW_NAME, cv2.WND_PROP_FULLSCREEN)
+            cv2.namedWindow(
+                WINDOW_NAME, cv2.WINDOW_GUI_NORMAL | cv2.WND_PROP_FULLSCREEN
+            )
             cv2.setWindowProperty(
                 WINDOW_NAME, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN
             )
         else:
-            cv2.namedWindow(
-                WINDOW_NAME,
-                cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_FREERATIO | cv2.WINDOW_NORMAL,
-            )
-            cv2.resizeWindow(WINDOW_NAME, *resolution)
-
-    def resize_frame_to_screen(self, frame: np.ndarray):
-        """
-        Resizes a given frame to match screen dimensions.
-
-        Only works on Windows.
-        """
-        assert self.is_windows, "Only Windows platforms need this method"
-        frame_height, frame_width = frame.shape[:2]
-
-        scale_height = self.screen_height / frame_height
-        scale_width = self.screen_width / frame_width
-
-        scale = min(scale_height, scale_width)
-
-        return cv2.resize(frame, (int(scale * frame_height), int(scale * frame_width)))
+            cv2.namedWindow(WINDOW_NAME, self.window_flags)
+            cv2.resizeWindow(WINDOW_NAME, *self.resolution)
 
     @property
     def current_presentation(self) -> Presentation:
-        """Returns the current presentation"""
+        """Returns the current presentation."""
         return self.presentations[self.current_presentation_index]
 
     def run(self):
@@ -341,10 +343,25 @@ class Display:
         self.lag = now() - self.last_time
         self.last_time = now()
 
+        if not self.record_to is None:
+            pres = self.current_presentation
+            self.recordings.append(
+                (pres.current_file, pres.current_frame_number, pres.fps)
+            )
+
         frame = self.lastframe
 
-        if self.is_windows and self.fullscreen:
-            frame = self.resize_frame_to_screen(frame)
+        # If Window was manually closed (impossible in fullscreen),
+        # we reopen it
+        if cv2.getWindowProperty(WINDOW_NAME, cv2.WND_PROP_VISIBLE) < 1:
+            cv2.namedWindow(WINDOW_NAME, self.window_flags)
+            cv2.resizeWindow(WINDOW_NAME, *self.resolution)
+
+        if WINDOWS:  # Only resize on Windows
+            _, _, w, h = cv2.getWindowImageRect(WINDOW_NAME)
+
+            if (h, w) != frame.shape[:2]:  # Only if shape is different
+                frame = cv2.resize(frame, (w, h), self.interpolation_flag)
 
         cv2.imshow(WINDOW_NAME, frame)
 
@@ -426,6 +443,35 @@ class Display:
     def quit(self):
         """Destroys all windows created by presentations and exits gracefully."""
         cv2.destroyAllWindows()
+
+        if not self.record_to is None and len(self.recordings) > 0:
+            file, frame_number, fps = self.recordings[0]
+
+            cap = cv2.VideoCapture(file)
+            cap.set(cv2.CAP_PROP_POS_FRAMES, frame_number - 1)
+            _, frame = cap.read()
+
+            w, h = frame.shape[:2]
+            fourcc = cv2.VideoWriter_fourcc(*"XVID")
+            out = cv2.VideoWriter(self.record_to, fourcc, fps, (h, w))
+
+            out.write(frame)
+
+            for _file, frame_number, _ in tqdm(
+                self.recordings[1:], desc="Creating recording file", leave=False
+            ):
+                if file != _file:
+                    cap.release()
+                    file = _file
+                    cap = cv2.VideoCapture(_file)
+
+                cap.set(cv2.CAP_PROP_POS_FRAMES, frame_number - 1)
+                _, frame = cap.read()
+                out.write(frame)
+
+            cap.release()
+            out.release()
+
         self.exit = True
 
 
@@ -435,6 +481,7 @@ class Display:
     default=FOLDER_PATH,
     type=click.Path(exists=True, file_okay=False),
     help="Set slides folder.",
+    show_default=True,
 )
 @click.help_option("-h", "--help")
 def list_scenes(folder):
@@ -468,6 +515,7 @@ def _list_scenes(folder) -> List[str]:
     default=FOLDER_PATH,
     type=click.Path(exists=True, file_okay=False),
     help="Set slides folder.",
+    show_default=True,
 )
 @click.option("--start-paused", is_flag=True, help="Start paused.")
 @click.option("--fullscreen", is_flag=True, help="Fullscreen mode.")
@@ -477,15 +525,38 @@ def _list_scenes(folder) -> List[str]:
     help="Skip all slides, useful the test if slides are working.",
 )
 @click.option(
+    "-r",
     "--resolution",
     type=(int, int),
-    default=(1280, 720),
+    default=(1920, 1080),
     help="Window resolution used if fullscreen is not set. You may manually resize the window afterward.",
     show_default=True,
 )
+@click.option(
+    "-i",
+    "--interpolation-flag",
+    type=click.Choice(INTERPOLATION_FLAGS.keys(), case_sensitive=False),
+    default="linear",
+    help="Set the interpolation flag to be used when resizing image. See OpenCV cv::InterpolationFlags.",
+    show_default=True,
+)
+@click.option(
+    "--record-to",
+    type=click.Path(dir_okay=False),
+    default=None,
+    help="If set, the presentation will be recorded into a AVI video file with given name.",
+)
 @click.help_option("-h", "--help")
 def present(
-    scenes, config_path, folder, start_paused, fullscreen, skip_all, resolution
+    scenes,
+    config_path,
+    folder,
+    start_paused,
+    fullscreen,
+    skip_all,
+    resolution,
+    interpolation_flag,
+    record_to,
 ):
     """Present the different scenes."""
 
@@ -545,6 +616,13 @@ def present(
     else:
         config = Config()
 
+    if not record_to is None:
+        _, ext = os.path.splitext(record_to)
+        if ext.lower() != ".avi":
+            raise click.UsageError(
+                f"Recording only support '.avi' extension. For other video formats, please convert the resulting '.avi' file afterwards."
+            )
+
     display = Display(
         presentations,
         config=config,
@@ -552,5 +630,7 @@ def present(
         fullscreen=fullscreen,
         skip_all=skip_all,
         resolution=resolution,
+        interpolation_flag=INTERPOLATION_FLAGS[interpolation_flag],
+        record_to=record_to,
     )
     display.run()
